"use strict";
var channel_1 = require('../../channel');
var fielddef_1 = require('../../fielddef');
var common_1 = require('../common');
var area;
(function (area) {
    function markType() {
        return 'area';
    }
    area.markType = markType;
    function properties(model) {
        var p = {};
        var config = model.config();
        var _orient = orient(config.mark.orient);
        if (_orient) {
            p.orient = _orient;
        }
        p.x = x(model.encoding().x, model.scaleName(channel_1.X), model.stack());
        var _x2 = x2(model.encoding().x, model.scaleName(channel_1.X), model.stack(), config.mark.orient);
        if (_x2) {
            p.x2 = _x2;
        }
        p.y = y(model.encoding().y, model.scaleName(channel_1.Y), model.stack());
        var _y2 = y2(model.encoding().y, model.scaleName(channel_1.Y), model.stack(), config.mark.orient);
        if (_y2) {
            p.y2 = _y2;
        }
        common_1.applyColorAndOpacity(p, model);
        common_1.applyMarkConfig(p, model, ['interpolate', 'tension']);
        return p;
    }
    area.properties = properties;
    function orient(orient) {
        if (orient) {
            return { value: orient };
        }
        return undefined;
    }
    function x(fieldDef, scaleName, stack) {
        if (stack && channel_1.X === stack.fieldChannel) {
            return {
                scale: scaleName,
                field: fielddef_1.field(fieldDef, { suffix: '_start' })
            };
        }
        else if (fielddef_1.isMeasure(fieldDef)) {
            return { scale: scaleName, field: fielddef_1.field(fieldDef) };
        }
        else if (fielddef_1.isDimension(fieldDef)) {
            return {
                scale: scaleName,
                field: fielddef_1.field(fieldDef, { binSuffix: '_mid' })
            };
        }
        return undefined;
    }
    function x2(fieldDef, scaleName, stack, orient) {
        if (orient === 'horizontal') {
            if (stack && channel_1.X === stack.fieldChannel) {
                return {
                    scale: scaleName,
                    field: fielddef_1.field(fieldDef, { suffix: '_end' })
                };
            }
            else {
                return {
                    scale: scaleName,
                    value: 0
                };
            }
        }
        return undefined;
    }
    function y(fieldDef, scaleName, stack) {
        if (stack && channel_1.Y === stack.fieldChannel) {
            return {
                scale: scaleName,
                field: fielddef_1.field(fieldDef, { suffix: '_start' })
            };
        }
        else if (fielddef_1.isMeasure(fieldDef)) {
            return {
                scale: scaleName,
                field: fielddef_1.field(fieldDef)
            };
        }
        else if (fielddef_1.isDimension(fieldDef)) {
            return {
                scale: scaleName,
                field: fielddef_1.field(fieldDef, { binSuffix: '_mid' })
            };
        }
        return undefined;
    }
    function y2(fieldDef, scaleName, stack, orient) {
        if (orient !== 'horizontal') {
            if (stack && channel_1.Y === stack.fieldChannel) {
                return {
                    scale: scaleName,
                    field: fielddef_1.field(fieldDef, { suffix: '_end' })
                };
            }
            else {
                return {
                    scale: scaleName,
                    value: 0
                };
            }
        }
        return undefined;
    }
    function labels(model) {
        return undefined;
    }
    area.labels = labels;
})(area = exports.area || (exports.area = {}));
//# sourceMappingURL=area.js.map